/****************************************************************************
**  CUBE        http://www.score-p.org/                                    **
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2014                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2009-2014                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/

/**
 * \file
 * \brief Provides an interface for simple caching mechanisms in metrics, using three maps
 */

#ifndef __SIMPLE_CACHE_H
#define __SIMPLE_CACHE_H


#include <map>
#include <stdint.h>

#include "CubeCache.h"


namespace cube
{
typedef int64_t simple_cache_key_t;
typedef int64_t sysres_id_t;
typedef int64_t cnode_id_t;

/**
 * Class "SimpleCache" is a class, which stores all values without any evaluation of probability
 * Drawback: memory footprint is too big. But this simple realization allows to test another functionalities
 */
class SimpleCache : public Cache
{
private:
    typedef std::map<simple_cache_key_t, Value*> cache_map;

    cnode_id_t  number_cnodes;
    sysres_id_t number_loc;

    CalculationFlavour myf;

    simple_cache_key_t threshold;

    cache_map stn_container;
    cache_map sum_container;

    simple_cache_key_t
    get_key( Cnode *, CalculationFlavour, Sysres *, CalculationFlavour );


public:

    SimpleCache( cnode_id_t         _n_cnode,
                 sysres_id_t        _n_loc,
                 CalculationFlavour metric_flavor = CUBE_CALCULATE_EXCLUSIVE
                 ) : number_cnodes( _n_cnode ), number_loc( _n_loc ),
                     myf( metric_flavor ), threshold( 0.7 * number_cnodes )
    {
    }

    virtual
    ~SimpleCache();

    virtual Value*
    getCachedValue( Cnode*             cnode,
                    CalculationFlavour cf,
                    Sysres*            sysres = 0,
                    CalculationFlavour sf = CUBE_CALCULATE_INCLUSIVE );                                                                                    ///< Returns Value or NULL, if not present

    virtual void
    setCachedValue( Value*,
                    Cnode*             cnode,
                    CalculationFlavour cf,
                    Sysres*            sysres = 0,
                    CalculationFlavour sf = CUBE_CALCULATE_INCLUSIVE );                                                                                ///< Stores or not (if not needed) a Value.
};
}

#endif
